/*
Copyright 2020 The SuperEdge Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1beta1

import (
	"context"
	"time"

	v1beta1 "github.com/superedge/superedge/pkg/penetrator/apis/nodetask.apps.superedge.io/v1beta1"
	scheme "github.com/superedge/superedge/pkg/penetrator/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// NodeTasksGetter has a method to return a NodeTaskInterface.
// A group's client should implement this interface.
type NodeTasksGetter interface {
	NodeTasks() NodeTaskInterface
}

// NodeTaskInterface has methods to work with NodeTask resources.
type NodeTaskInterface interface {
	Create(ctx context.Context, nodeTask *v1beta1.NodeTask, opts v1.CreateOptions) (*v1beta1.NodeTask, error)
	UpdateStatus(ctx context.Context, nodeTask *v1beta1.NodeTask, opts v1.UpdateOptions) (*v1beta1.NodeTask, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1beta1.NodeTask, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1beta1.NodeTaskList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.NodeTask, err error)
	NodeTaskExpansion
}

// nodeTasks implements NodeTaskInterface
type nodeTasks struct {
	client rest.Interface
}

// newNodeTasks returns a NodeTasks
func newNodeTasks(c *NodestaskV1beta1Client) *nodeTasks {
	return &nodeTasks{
		client: c.RESTClient(),
	}
}

// Get takes name of the nodeTask, and returns the corresponding nodeTask object, and an error if there is any.
func (c *nodeTasks) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1beta1.NodeTask, err error) {
	result = &v1beta1.NodeTask{}
	err = c.client.Get().
		Resource("nodetasks").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of NodeTasks that match those selectors.
func (c *nodeTasks) List(ctx context.Context, opts v1.ListOptions) (result *v1beta1.NodeTaskList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1beta1.NodeTaskList{}
	err = c.client.Get().
		Resource("nodetasks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested nodeTasks.
func (c *nodeTasks) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("nodetasks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a nodeTask and creates it.  Returns the server's representation of the nodeTask, and an error, if there is any.
func (c *nodeTasks) Create(ctx context.Context, nodeTask *v1beta1.NodeTask, opts v1.CreateOptions) (result *v1beta1.NodeTask, err error) {
	result = &v1beta1.NodeTask{}
	err = c.client.Post().
		Resource("nodetasks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(nodeTask).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *nodeTasks) UpdateStatus(ctx context.Context, nodeTask *v1beta1.NodeTask, opts v1.UpdateOptions) (result *v1beta1.NodeTask, err error) {
	result = &v1beta1.NodeTask{}
	err = c.client.Put().
		Resource("nodetasks").
		Name(nodeTask.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(nodeTask).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the nodeTask and deletes it. Returns an error if one occurs.
func (c *nodeTasks) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Resource("nodetasks").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *nodeTasks) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("nodetasks").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched nodeTask.
func (c *nodeTasks) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.NodeTask, err error) {
	result = &v1beta1.NodeTask{}
	err = c.client.Patch(pt).
		Resource("nodetasks").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
